import { AnyAction } from '@reduxjs/toolkit'
import { act, render, screen } from '@testing-library/react'
import userEvent from '@testing-library/user-event'
import { FluidType } from 'enums'
import { DateTime } from 'luxon'
import { PerformanceIndicator } from 'models'
import React from 'react'
import { Provider } from 'react-redux'
import { createMockEcolyoStore, mockAnalysisState } from 'tests/__mocks__/store'
import FluidPerformanceIndicator from './FluidPerformanceIndicator'

const monthlyStore = createMockEcolyoStore()
const yearlyStore = createMockEcolyoStore({
  analysis: { ...mockAnalysisState, period: 'year' },
})
const mockedNavigate = jest.fn()
jest.mock('react-router-dom', () => ({
  ...jest.requireActual('react-router-dom'),
  useNavigate: () => mockedNavigate,
}))

const elecPerformanceIndicator: PerformanceIndicator = {
  value: 100,
  compareValue: 90,
  percentageVariation: 0.1,
}

const waterPerformanceIndicator: PerformanceIndicator = {
  value: 140,
  compareValue: 150,
  percentageVariation: -0.1,
}

const comparisonDate = DateTime.local(2024, 4, 15)

describe('FluidPerformanceIndicator component', () => {
  it('should render elec indicators correctly and navigate on click', async () => {
    const dispatchSpy = jest.spyOn(monthlyStore, 'dispatch')
    const setShowCompareMock = jest.fn()
    dispatchSpy.mockImplementation((action: AnyAction) => {
      if (action.type === 'chart/setShowCompare') {
        setShowCompareMock(action.payload)
      }
      return action
    })

    const { container } = render(
      <Provider store={monthlyStore}>
        <FluidPerformanceIndicator
          performanceIndicator={elecPerformanceIndicator}
          fluidType={FluidType.ELECTRICITY}
          comparisonDate={comparisonDate}
        />
      </Provider>
    )

    expect(screen.getByTestId('fluid-type-ELECTRICITY')).toHaveTextContent(
      'ELECTRICITY'
    )
    expect(screen.getByTestId('fluid-value-ELECTRICITY')).toHaveTextContent(
      '100'
    )
    expect(
      screen.getByTestId('fluid-comparison-ELECTRICITY')
    ).toHaveTextContent('+10,00 % / avril 2024')

    await act(async () => {
      await userEvent.click(screen.getByRole('listitem'))
    })

    expect(mockedNavigate).toHaveBeenCalledWith('/consumption/electricity')
    expect(dispatchSpy).toHaveBeenCalledWith({
      type: 'chart/setShowCompare',
      payload: false,
    })

    expect(container).toMatchSnapshot()
  })

  it('should render water indicators correctly and dispatch setShowCompare year on click', async () => {
    const dispatchSpy = jest.spyOn(yearlyStore, 'dispatch')
    const setShowCompareMock = jest.fn()
    dispatchSpy.mockImplementation((action: AnyAction) => {
      if (action.type === 'chart/setShowCompare') {
        setShowCompareMock(action.payload)
      }
      return action
    })

    render(
      <Provider store={yearlyStore}>
        <FluidPerformanceIndicator
          performanceIndicator={waterPerformanceIndicator}
          fluidType={FluidType.WATER}
          comparisonDate={comparisonDate}
        />
      </Provider>
    )

    expect(screen.getByTestId('fluid-type-WATER')).toHaveTextContent('WATER')
    expect(screen.getByTestId('fluid-value-WATER')).toHaveTextContent('140')
    expect(screen.getByTestId('fluid-comparison-WATER')).toHaveTextContent(
      '-10,00 % / avril 2024'
    )

    await act(async () => {
      await userEvent.click(screen.getByRole('listitem'))
    })

    expect(mockedNavigate).toHaveBeenCalledWith('/consumption/water')
    expect(dispatchSpy).toHaveBeenCalledWith({
      type: 'chart/setShowCompare',
      payload: true,
    })
  })

  it('should render "no data" message when value is not available', () => {
    const performanceIndicator: PerformanceIndicator = {
      value: null,
      compareValue: null,
      percentageVariation: null,
    }
    const { container } = render(
      <Provider store={monthlyStore}>
        <FluidPerformanceIndicator
          performanceIndicator={performanceIndicator}
          fluidType={FluidType.WATER}
          comparisonDate={comparisonDate}
        />
      </Provider>
    )

    expect(container).toMatchSnapshot()
    expect(
      screen.getByText('performance_indicator.fpi.no_data')
    ).toBeInTheDocument()
  })
})
