import { act, render, screen } from '@testing-library/react'
import { userEvent } from '@testing-library/user-event'
import ProfileTypeOptions from 'components/Options/ProfileTypeOptions/ProfileTypeOptions'
import {
  HousingType,
  IndividualInsulationWork,
  IndividualOrCollective,
  ThreeChoicesAnswer,
} from 'enums'
import React from 'react'
import { Provider } from 'react-redux'
import { mockProfileType } from 'tests/__mocks__/profileType.mock'
import {
  createMockEcolyoStore,
  mockChallengeState,
  mockProfileState,
} from 'tests/__mocks__/store'

const mockedNavigate = jest.fn()
jest.mock('react-router-dom', () => ({
  ...jest.requireActual('react-router-dom'),
  useNavigate: () => mockedNavigate,
}))

describe('ProfileTypeOptions component', () => {
  const store = createMockEcolyoStore()
  it('should be rendered correctly with profile NOT completed', async () => {
    const { container } = render(
      <Provider store={store}>
        <ProfileTypeOptions />
      </Provider>
    )
    expect(container.getElementsByClassName('profile-icon').length).toBeTruthy()
    await act(async () => {
      await userEvent.click(screen.getByText('profile_type.read_profile'))
    })
    expect(mockedNavigate).toHaveBeenCalled()
  })
  it('should be rendered when user complete profile type form', () => {
    const store = createMockEcolyoStore({
      profile: { ...mockProfileState, isProfileTypeCompleted: true },
      profileType: mockProfileType,
      challenge: mockChallengeState,
    })
    const { container } = render(
      <Provider store={store}>
        <ProfileTypeOptions />
      </Provider>
    )
    expect(
      container.getElementsByClassName('profile-container')[0]
    ).toBeInTheDocument()
  })
  it('should be rendered when housing_type = apartment', () => {
    const store = createMockEcolyoStore({
      profile: { ...mockProfileState, isProfileTypeCompleted: true },
      profileType: { ...mockProfileType, housingType: HousingType.APARTMENT },
      challenge: mockChallengeState,
    })
    const { container } = render(
      <Provider store={store}>
        <ProfileTypeOptions />
      </Provider>
    )
    expect(container.getElementsByClassName('floor')[0]).toBeInTheDocument()
  })
  it('should display heating with equipments', () => {
    const store = createMockEcolyoStore({
      profile: { ...mockProfileState, isProfileTypeCompleted: true },
      profileType: {
        ...mockProfileType,
        heating: IndividualOrCollective.INDIVIDUAL,
        housingType: HousingType.APARTMENT,
        hasInstalledVentilation: ThreeChoicesAnswer.YES,
        hasReplacedHeater: ThreeChoicesAnswer.YES,
      },
      challenge: mockChallengeState,
    })
    const { container } = render(
      <Provider store={store}>
        <ProfileTypeOptions />
      </Provider>
    )
    expect(
      container.getElementsByClassName('equipments')[0]
    ).toBeInTheDocument()
  })
  it('should display insulation work', () => {
    const store = createMockEcolyoStore({
      profile: {
        ...mockProfileState,
        isProfileTypeCompleted: true,
      },
      profileType: {
        ...mockProfileType,
        housingType: HousingType.APARTMENT,
        individualInsulationWork: [
          IndividualInsulationWork.ROOF_INSULATION,
          IndividualInsulationWork.WINDOW_REPLACEMENT,
        ],
      },
      challenge: mockChallengeState,
    })
    const { container } = render(
      <Provider store={store}>
        <ProfileTypeOptions />
      </Provider>
    )
    expect(
      container.getElementsByClassName('insulation')[0]
    ).toBeInTheDocument()
  })
})
